#include "AlgoRunTestVehicleScenario.h"
#include <cmath>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "carte.h"
#include "Path.h"
#include "ScenarioPermutation.h"
#include "ScenarioLoader.h"
#include "ScenarioData.h"
#include "ISystemSolver.h"
#include "SystemSolution.h"
#include "VehicleSolution.h"



AlgoRunTestVehicleScenario::AlgoRunTestVehicleScenario()
{
}


AlgoRunTestVehicleScenario::~AlgoRunTestVehicleScenario()
{
}

int AlgoRunTestVehicleScenario::Run(int argc, const char ** argv)
{
	//cin.get();

	string fileNameCarte = "quebec-carte.txt";
	string fileNameBorne = "quebec-bornes.txt";
	string fileNameScenario = ScenarioLoader::NO_SCENARIO;
	string modeName = "interactive";


	bool bActivationState[Scenario::eMAX_SOLVERTYPE];// = { true,true,true,true,true,true,true };
	bActivationState[Scenario::eINIT]						= true ;
	bActivationState[Scenario::eNO_COOP]					= true ;
	bActivationState[Scenario::eNO_COOP_WAIT]				= true ;
	bActivationState[Scenario::eHCA_FI]						= true ;
	bActivationState[Scenario::eHCA_CASCADE]				= true ;
	bActivationState[Scenario::eHCA_PERMUTATION]			= false ;
	bActivationState[Scenario::eWHCA_SMALL_FI]				= true ;
	bActivationState[Scenario::eWHCA_MEDIUM_FI]				= true ;
	bActivationState[Scenario::eWHCA_LARGE_FI]				= true ;
	bActivationState[Scenario::eWHCA_SMALL_CASCADE]			= true;
	bActivationState[Scenario::eWHCA_MEDIUM_CASCADE]		= true;
	bActivationState[Scenario::eWHCA_LARGE_CASCADE]			= true;
	bActivationState[Scenario::eWHCA_SMALL_PERMUTATION]		= false;
	bActivationState[Scenario::eWHCA_MEDIUM_PERMUTATION]	= false;
	bActivationState[Scenario::eWHCA_LARGE_PERMUTATION]		= false;


	//Arguments
	//uqam-carte.txt bornes.txt scenario.txt scenario_viz

	if (argc > 1)
	{
		fileNameCarte = argv[1];
	}

	if (argc > 2)
	{
		fileNameBorne = argv[2];
	}

	if (argc > 3)
	{
		fileNameScenario = argv[3];
	}

	if (argc > 4)
	{
		modeName = argv[4];
	}

	//if (argc < 4) {
	//	cout << "./coroadperation carte.txt bornes.txt scenario.txt" << endl;
	//	return 1;
	//}
	Carte carte;
	{
		ifstream iscarte(fileNameCarte);
		if (iscarte.fail()) {
			cerr << "Erreur d'ouverture du fichier carte: '" << fileNameCarte << "' !" << endl;
			return 2;
		}
		iscarte >> carte;
		cerr << "Carte charge!" << endl;
	}
	{
		ifstream isbornes(fileNameBorne);
		if (isbornes.fail()) {
			cerr << "Erreur d'ouverture du fichier bornes: '" << fileNameBorne << "' !" << endl;
			return 2;
		}
		while (isbornes) {
			string nom;
			char L;
			int niveau;
			Point point;
			isbornes >> nom;
			if (!isbornes || nom.empty()) break;
			isbornes >> L >> niveau >> point >> ws;
			assert(L == 'L');
			cerr << "."; cerr.flush();
			carte.ajouterBorne(nom, point, niveau);
		}
		cerr << "Bornes charges!" << endl;
	}

	int interactiveMode = 1;
	if (modeName.compare("interactive") == 0)
	{
		interactiveMode = 1;
	}
	else if (modeName.compare("scenario_viz") == 0)
	{
		interactiveMode = 2;
	}

	IScenarioLoader* scenarioLoader = new ScenarioLoader(fileNameScenario, carte);
	IScenario* scenario = new Scenario(scenarioLoader,carte); //errr for the Carte dependency here...
	scenario->SetActivation(bActivationState);
	scenario->SolveAllVehicle();//TODO: should have single interface for adding vehicle and solving all systems
	scenario->SolveAllSystem();
	Scenario::SolverType focusSolver = Scenario::eNO_COOP;
	if (interactiveMode == 1)
	{
		while (cin) {
			cin >> ws;
			if (cin.fail() || cin.eof())
				break;
			
			scenario->LoadNamelessVehicle(cin);
			cerr << "."; cerr.flush();
			std::cout << std::setprecision(8);
			scenario->SolveNextVehicle();
			VehicleSolution* last = scenario->GetSystemSolver(focusSolver)->GetCurrentSystemSolution()->GetLastVehicleSolution();

			cout << last->toStringInteractive();
			cin.get();
			//cout << endl;
		}
	}
	else if (interactiveMode == 2)
	{
		cout << scenario->toString() << endl;
		cout << scenario->toStringPerformance() << endl;
		//cout << endl << scenario->toStringInteractive();
	}
	else
	{
		cout << endl << scenario->toString();
	}

	cin.get();

	//Clean all
	delete scenario;
	delete scenarioLoader;

	return 0;

}
