#include "ElectricTerminal.h"

#include "ElectricReservation.h"
#include <vector>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <algorithm>


setER::iterator ElectricTerminal::Find(TimeUnit timeStamp, setER& in_SearchIn, SearchMethodReservation in_SearchMethod)
{
	//Temporary reservation list to test
	ElectricReservation tempErl(timeStamp,nullptr);

	setER::iterator it = in_SearchIn.end();
	it = (in_SearchIn.*in_SearchMethod)(&tempErl);
	return it;
}


ElectricTerminal::ElectricTerminal(Borne & in_RefBorne)
:mRefBorne(in_RefBorne)
{
}

ElectricTerminal::ElectricTerminal(const ElectricTerminal & otherTerminal)
{
	//Empty data
	for each (ElectricReservationPtr var in mOrderedSetReservation)
	{
		delete var;
	}

	mOrderedSetReservation.clear();

	//Copy data
	this->mRefBorne = otherTerminal.mRefBorne;
	for each (ElectricReservationPtr var in otherTerminal.mOrderedSetReservation)
	{
		mOrderedSetReservation.insert(new ElectricReservation(*var));
	}
}


ElectricTerminal::~ElectricTerminal()
{
	for each (ElectricReservationPtr var in mOrderedSetReservation)
	{
		delete var;
	}

	mOrderedSetReservation.clear();
}

bool ElectricTerminal::TryReserve(ElectricReservation * er)
{
	bool success = true;
	try {
		std::vector<ElectricReservation*> overlap;
		ReservationOverlap(er, overlap);
		if (overlap.empty())
		{
			mOrderedSetReservation.insert(er);
		}
		else
		{
			success = false;
		}

	}
	catch (...)
	{
		success = false;
	}

	if (!success)
	{
		cout << "WARNING TryReserve failed for " << *er << endl;
	}
	//cout << *this << endl;

	return success;
}

bool ElectricTerminal::TryFree(ElectricReservation * er)
{
	bool success = true;
	try {
		setER::iterator it = Find(er->GetTimeRange().mStart, mOrderedSetReservation, SET_OPERATOR_RESERVATION(find));

		if (it != mOrderedSetReservation.end())
		{
			mOrderedSetReservation.erase(it);
		}
		else
		{
			success = false;
		}

	}
	catch (...)
	{
		success = false;
	}
	return success;
}

void ElectricTerminal::ReservationOverlap(ElectricReservation* er, std::vector<ElectricReservation*>& out_VecOverlapingReservation)
{
	ReservationOverlap(er->GetTimeRange(), out_VecOverlapingReservation);
}

void ElectricTerminal::ReservationOverlap(TimeRange tr, std::vector<ElectricReservation*>& out_VecOverlapingReservation)
{
	//cout << endl << endl;
	//cout << "TESTING OVERLAP  FOR " << tr << endl;
	//cout << "Terminal state is " << endl << *this << endl;
	//ElectricReservation tempStart(tr, nullptr);
	//TimeRange endRange(tr.mEnd,tr.mEnd + tr.Duration());
	//ElectricReservation tempEnd(endRange, nullptr);

	//cout << "TEMP RESERVATIONS ARE" << endl;
	//cout << tempStart << endl;
	//cout << tempEnd << endl;

	//setER::iterator itSoonest = mOrderedSetReservation.lower_bound(&tempStart);
	//setER::iterator itLatest = mOrderedSetReservation.lower_bound(&tempEnd);

	//if (itSoonest != mOrderedSetReservation.end())
	//{
	//	cout << "Soonest " << *itSoonest << endl;
	//}
	//if (itLatest != mOrderedSetReservation.end())
	//{
		//cout << "Latest" << *itLatest << endl;
		//itLatest++;
	//}

	//TODO: we have a iterator issue we do not handle properly... checking all the entries from the start
	for (setER::iterator it = mOrderedSetReservation.begin(); it != mOrderedSetReservation.end(); ++it)
	{
		//We have an overlap!
		TimeRange test = (*it)->GetTimeRange();
		//cout << "Overlap potential" << *it << endl;
		if ( test && tr)
		{
			//cout << "Overlap confirmed" << *it << endl;
			out_VecOverlapingReservation.push_back(*it);
		}
	}
}

TimeRange ElectricTerminal::FindNextAvailableReservation(TimeRange in_IdealRange)
{
	std::vector<ElectricReservation*> vOverlappingBuffer;
	TimeRange returnRange = in_IdealRange;
	TimeUnit duration = returnRange.Duration();

	bool bSearching = true;
	while (bSearching)
	{
		vOverlappingBuffer.clear();
		ReservationOverlap(returnRange, vOverlappingBuffer);
		if (vOverlappingBuffer.empty())
		{
			//We found it!
			bSearching = false;
			break;
		}

		TimeUnit highestEnd = 0;
		//We should continue the search at the end of the last one...
		for each (auto var in vOverlappingBuffer)
		{
			highestEnd = std::max(highestEnd,var->GetTimeRange().mEnd);
		}
		returnRange = TimeRange(highestEnd, highestEnd + duration);
	}
	//cout << "RESERVATION RANGE RETURNED " << returnRange <<  endl;
	return returnRange;
}


string ElectricTerminal::toString() const
{
	std::ostringstream ss;
	ss << setw(20) << left << "Terminal name" << setw(20) << left << GetName() << endl;
	ss << "Reservations" << endl;
	for each (auto var in mOrderedSetReservation)
	{
		ss << *var << endl;
	}

	return ss.str();
}

ostream & operator<<(ostream & os, const ElectricTerminal & dt)
{
	os << dt.toString();
	return os;
}
