
#include "IScenario.h"
#include "ISystemSolver.h"
#include "IVehicleSolver.h"
#include "IObjectiveFunction.h"
#include "SystemSolution.h"
#include "ScenarioData.h"
#include "ElectricCircuit.h"
#include "IScenarioLoader.h"
#include "PerformanceAnalysis.h"
#include "IArrangement.h"

#include <sstream>
#include <istream>

IScenario::~IScenario()
{
	for each (auto var in mSystemSolver)
	{
		delete var;
	}
	mSystemSolver.clear();

	for each (auto var in mVehicleSolver)
	{
		delete var;
	}
	mVehicleSolver.clear();

	for each (auto var in mObjectiveFunction)
	{
		delete var;
	}
	mObjectiveFunction.clear();

	for each (auto var in mSolverCircuit)
	{
		delete var;
	}
	mSolverCircuit.clear();

	for each(auto var in mArrangement)
	{
		delete var;
	}
	mArrangement.clear();

	//delete mRefScenarioLoader;
}

bool IScenario::SolveNextVehicle()
{
	bool bCanSolve = mDataIndexSimulator < GetRefData()->mVecVehicle.size();
	if (bCanSolve)
	{
		for each (auto var in mSystemSolver)
		{
			var->AddVehicle(GetRefData()->mVecVehicle[mDataIndexSimulator]);
		}
		mDataIndexSimulator++;
	}
	return bCanSolve;
}

void IScenario::SolveAllVehicle()
{
	while (SolveNextVehicle());
}

void IScenario::SolveAllSystem()
{
	for each (auto var in mSystemSolver)
	{
		var->SolveSystem();
	}
}

void IScenario::SetActivation(bool toActivate[])
{
	for (unsigned int i = 0 ; i < mSystemSolver.size(); ++i)
	{
		mSystemSolver[i]->SetActivation(toActivate[i]);
	}
}

void IScenario::LoadNamelessVehicle(std::istream & inDataVehicle)
{
	mRefScenarioLoader->LoadNextNamelessVehicle(inDataVehicle, GetRefData());
}

std::string IScenario::toString() const
{
	std::ostringstream ss;
	ss << GetRefData()->toString();
	for each (auto var in mSystemSolver)
	{
		 ss << (*(var->GetCurrentSystemSolution()));
	}
	return ss.str();
}

std::string IScenario::toStringCost() const
{
	std::ostringstream ss;
	ss << "Cost for each" << endl;
	for each (auto var in mSystemSolver)
	{
		ss << var->GetCurrentSystemSolution()->mSystemObjectiveFunctionValue << endl;
	}
	return ss.str();
}

std::string IScenario::toStringPerformance() const
{
	std::ostringstream ss;
	ss << "Performance for each" << endl;
	for each (auto var in mSystemSolver)
	{
		ss << *var->GetPerformanceAnalysis() << endl;
	}
	return ss.str();
}

std::string IScenario::toStringInteractive() const
{
	return GetRefData()->toStringInteractive();
}
