#pragma once

class Vehicle;
class ElectricCircuit;
class SystemSolution;
class IObjectiveFunction;
class IVehicleSolver;
class ScenarioData;
class PerformanceAnalysis;


class ISystemSolver
{
protected:
	IObjectiveFunction* mObjectiveFunction = nullptr;
	IVehicleSolver* mVehicleSolver = nullptr;

	//Solver has it's own data internally
	ScenarioData* mSolverData = nullptr;

	PerformanceAnalysis* mPerformanceAnalysis = nullptr;

	bool mIsActivated = true;

protected:
	IObjectiveFunction* GetObjectiveFunction() const { return mObjectiveFunction; }
	IVehicleSolver* GetVehicleSolver() const {return mVehicleSolver;}
	//These will solve when adding and removing candidates
	virtual void AddVehicleInternal(Vehicle* newVehicle) = 0;
	virtual void RemoveVehicleInternal(Vehicle* newVehicle) = 0;

public:
	ISystemSolver(IObjectiveFunction* inObjectiveFunction, IVehicleSolver* inVehicleSolver, ElectricCircuit* inElectricCircuit);
	virtual ~ISystemSolver();

	//These will solve when adding and removing candidates
	virtual void AddVehicle(Vehicle* newVehicle);
	virtual void RemoveVehicle(Vehicle* newVehicle);

	virtual void SolveSystem() = 0;

	void SetActivation(bool bActivation) { mIsActivated = bActivation; }

	virtual SystemSolution* GetCurrentSystemSolution() const = 0;

	//One should only modify optionnal parameters when getting ScenarioData
	//TODO: have a proxy?
	ScenarioData* ModifyOptionnalScenarioData() const { return mSolverData; }
	PerformanceAnalysis const * const GetPerformanceAnalysis() const { return mPerformanceAnalysis; }
};
