#include "SystemSolution.h"

#include "VehicleSolution.h"
#include "Vehicle.h"
#include <algorithm>
#include <assert.h>


void SystemSolution::AddSolution(VehicleSolution * newSolution)
{
	auto itSolutionVehicle = mMapVehicleSolution.find(newSolution->mRefVehicle);
	//Replace the solution
	if (itSolutionVehicle != mMapVehicleSolution.end())
	{
		auto itOrderedSolutionVehicle = find(mOrderedVehicleSolution.begin(), mOrderedVehicleSolution.end(), itSolutionVehicle->second);
		assert(itOrderedSolutionVehicle != mOrderedVehicleSolution.end());
		//TODO: should we update the order for solution?
		*itOrderedSolutionVehicle = newSolution;
		//Delete old solution
		delete itSolutionVehicle->second;
		//Update map
		itSolutionVehicle->second = newSolution;
	}
	else
	{
		//Create a new entry
		mOrderedVehicleSolution.push_back(newSolution);
		mMapVehicleSolution.insert({ newSolution->mRefVehicle,newSolution });
	}
}

bool SystemSolution::IsImpossible() const
{
	bool impossible = false;
	for each(auto var in mOrderedVehicleSolution)
	{
		if (var->IsImpossible())
		{
			impossible = true;
			break;
		}
	}
	return impossible;
}

SystemSolution::SystemSolution()
{
}


SystemSolution::~SystemSolution()
{
	for each (auto var in mMapVehicleSolution)
	{
		delete var.second;
	}

	mMapVehicleSolution.clear();
	mOrderedVehicleSolution.clear();
}

SystemSolution& SystemSolution::operator=(const SystemSolution & other)
{
	//Delete
	for each (auto var in mMapVehicleSolution)
	{
		delete var.second;
	}

	mMapVehicleSolution.clear();
	mOrderedVehicleSolution.clear();

	//Copy everything else
	this->mDescription = other.mDescription;
	this->mSystemObjectiveFunctionValue = other.mSystemObjectiveFunctionValue;
	for each(auto var in other.mOrderedVehicleSolution)
	{
		this->AddSolution(var->clone());
	}
	return *this;
}

SystemSolution * SystemSolution::clone() const
{
	return new SystemSolution(*this);
}

ostream & operator<<(ostream & os, const SystemSolution & dt)
{
	os << "----------------------------------------------------" << endl;
	os << "-------------------System solution------------------" << endl;
	os << "----------------------------------------------------" << endl;
	os << dt.mDescription << endl;
	os << "Total objective function: " << dt.mSystemObjectiveFunctionValue << endl;
	
	//Traverse in reverse order

	for (auto it = dt.mOrderedVehicleSolution.begin(); it != dt.mOrderedVehicleSolution.end(); ++it)
	{
		os << *(*it)->mRefVehicle << **it << endl;
		os << "******" << endl << endl;
	}
	return os;
}
