#include "SystemSolverCAStar.h"

#include "SystemSolverImmediate.h"
#include "SystemSolver.h"
#include "ISystemSolver.h"
#include "ElectricCircuit.h"
#include "ScenarioData.h"
#include "SystemSolution.h"
#include "IVehicleSolver.h"
#include "PerformanceAnalysis.h"
#include "IArrangement.h"
#include <assert.h>
#include <iostream>

SystemSolverCAStar::SystemSolverCAStar(IObjectiveFunction* inObjectiveFunction, IVehicleSolver* inVehicleSolver, IArrangement* inArrangment, ElectricCircuit* inElectricCircuit)
:SystemSolverArranged(inObjectiveFunction, inVehicleSolver, inArrangment, inElectricCircuit)
{
}


SystemSolverCAStar::~SystemSolverCAStar()
{
	ResetSolver();
}

void SystemSolverCAStar::SolveSystem()
{
	if (!mIsActivated)
	{
		return;
	}
	mPerformanceAnalysis->StartExperiment();

	ResetSolver();

	//std::cout << "Before " << mSolverData->mElectricCircuit->computeTotalReservation() << endl;

	//Find the best ABSOLUTE priority
	mArrangement->startSequence(&mSolverData->mVecVehicle);
	std::vector<Vehicle*>* curArrangement = mArrangement->GetNextSequence();
	while(curArrangement != nullptr)
	{
		//Create a new electric circuit from current state
		ElectricCircuit* simulatedCircuit = mSolverData->mElectricCircuit->clone();
		mVehicleSolver->ReplaceRefElectricCircuit(simulatedCircuit);
		SystemSolverImmediate* solverImmediate = new SystemSolverImmediate(mObjectiveFunction, mVehicleSolver, simulatedCircuit);
		ScenarioData* dataSolver = solverImmediate->ModifyOptionnalScenarioData();
		if (dataSolver->mObjOptionnalPartialSolution != nullptr)
		{
			dataSolver->mObjOptionnalPartialSolution = mSolverData->mObjOptionnalPartialSolution->clone();
		}
		dataSolver->mfOptTimeHorizon = mSolverData->mfOptTimeHorizon;
		//Add the vehicle according to current order previously generated
		for (unsigned int j = 0; j < curArrangement->size(); ++j)
		{
			solverImmediate->AddVehicle((*curArrangement)[j]);
		}

		//If we have a better solution keep the solver and order!
		if (solverImmediate->GetCurrentSystemSolution()->mSystemObjectiveFunctionValue < GetCurrentSystemSolution()->mSystemObjectiveFunctionValue)
		{
			ResetSolver();
			//mSystemSolution = solverImmediate->GetCurrentSystemSolution();
			mBestSolver = solverImmediate;
			mBestCircuit = simulatedCircuit;
		}
		else
		{
			//Otherwise, delete them
			delete solverImmediate;
			delete simulatedCircuit;
		}
		//Prepare next sequence
		curArrangement = mArrangement->GetNextSequence();
	};

	//assert(false);
	if (mBestSolver != nullptr)
	{
		mBestSolver->GetCurrentSystemSolution()->mDescription = mSystemSolution->mDescription;
	}
	mPerformanceAnalysis->PauseExperiment(GetCurrentSystemSolution()->mSystemObjectiveFunctionValue);
}